@@ -90,7 +90,7 @@ rank.mbnma <- function(x, params="auc", lower_better=FALSE, treats=NULL,
   checkmate::reportAssertions(argcheck)
 
   # Check level
-  if (level=="class" & !("classes" %in% names(x$network))) {
+  if (level=="class" & length(x$model.arg$class.effect)==0) {
     stop("`level` has been specified as `class` yet `x` is not a class effect model")
   }
   level <- ifelse(level=="treatment", "treatments", "classes")
@@ -208,6 +208,8 @@ rank.mbnma <- function(x, params="auc", lower_better=FALSE, treats=NULL,
 #' @inheritParams predict.mbnma
 #' @param ... further arguments passed to `knitr::kable`
 #'
+#' @return Prints summary details of the model results to the console
+#'
 #' @export
 summary.mbnma <- function(object, ...) {
   checkmate::assertClass(object, "mbnma")
@@ -242,23 +244,27 @@ summary.mbnma <- function(object, ...) {
 
 
 
-#' Predict responses over time in a given population based on MBNMA time-course
+#' Predict effects over time in a given population based on MBNMA time-course
 #' models
 #'
-#' Used to predict responses over time for different treatments or to predict
+#' Used to predict effects over time for different treatments or to predict
 #' the results of a new study. For MBNMA models that include consistency
 #' relative effects on time-course parameters, this is calculated by combining
 #' relative treatment effects with a given reference treatment response
 #' (specific to the population of interest).
 #'
+#' By default the network reference treatment baseline (`E0`) and time-course
+#' parameter values are set to zero so that `predict()` estimates mean differences
+#' (/relative treatment effects) over time versus the network reference treatment.
+#'
 #' @param object An S3 object of `class("mbnma")` generated by running
 #'   a time-course MBNMA model
 #' @param times A sequence of positive numbers indicating which time points to
-#'   predict mean responses for
+#'   predict mean responses for (or at which to conduct a node-split if used with `mb.nodesplit()`)
 #' @param E0 An object to indicate the value(s) to use for the response at time = 0
 #'   in the prediction. This can take a number of different formats depending
 #'   on how it will be used/calculated. The default is 0 but this may lead
-#'   to non-sensical predictions.
+#'   to non-sensical predictions if Ratio of Means are modeled.
 #'   * `numeric()` A single numeric value representing the deterministic response at time = 0
 #'   * `formula()` A formula representing a stochastic distribution for the response
 #'   at time = 0. This is specified as a random number generator
@@ -288,7 +294,8 @@ summary.mbnma <- function(object, ...) {
 #'   parameter modelled in `mbnma`. Their values can be either of the following:
 #'   * `numeric()` A numeric value representing the deterministic value of the time-course parameter in
 #'   question in individuals given the reference treatment. `0` is used as the default, which assumes no
-#'   effect of time on the reference treatment.
+#'   effect of time on the reference treatment (i.e. mean differences / relative effects versus the
+#'   reference treatment are modeled).
 #'   * `formula()` A formula representing a stochastic distribution for the value of the time-course
 #'   parameter in question. This is specified as a random number generator (RNG) given as a formula,
 #'   and can take any RNG distribution for which a function exists in R. For example: `~rnorm(n, -3, 0.2)`.
@@ -296,6 +303,7 @@ summary.mbnma <- function(object, ...) {
 #' @param synth A character object that can take the value `"common"` or `"random"` that
 #'   specifies the the type of pooling to use for synthesis of `ref.resp`. Using `"random"` rather
 #'   than `"common"` for `synth` will result in wider 95\\% CrI for predictions.
+#' @param lim Specifies calculation of either 95% credible intervals (`lim="cred"`) or 95% prediction intervals (`lim="pred"`).
 #' @param ... Arguments to be sent to R2jags for synthesis of the network
 #'   reference treatment effect (using [ref.synth()])
 #'
@@ -355,6 +363,7 @@ predict.mbnma <- function(object, times=seq(0, max(object$model.arg$jagsdata$tim
                           E0=0,
                           treats = NULL, level="treatment",
                           ref.resp=NULL, synth="common",
+                          lim="cred",
                           ...) {
   ######## CHECKS ########
 
@@ -365,11 +374,12 @@ predict.mbnma <- function(object, times=seq(0, max(object$model.arg$jagsdata$tim
                            sorted=TRUE, add=argcheck)
   checkmate::assertChoice(level, choices=c("treatment", "class"), add=argcheck)
   checkmate::assertChoice(synth, choices=c("random", "common"), add=argcheck)
+  checkmate::assertChoice(lim, choices=c("cred", "pred"), add=argcheck)
   #checkmate::assertClass(treats, classes=c("numeric", "character"), null.ok=TRUE, add=argcheck)
   checkmate::reportAssertions(argcheck)
 
-  if (object$model.arg$link=="rom") {
-    stop("'predict()' cannot currently be used with MBNMAs modelled using ratios of means (link='rom')")
+  if (object$model.arg$link=="log") {
+    stop("'predict()' cannot currently be used with MBNMAs modelled using ratios of means (link='log')")
   }
 
   # Check if level="class" that class effect model was fitted
@@ -388,9 +398,14 @@ predict.mbnma <- function(object, times=seq(0, max(object$model.arg$jagsdata$tim
     level <- "treatments"
   }
 
-  # Check whether UME has been used and stop if so
-  if (object[["model.arg"]][["UME"]]!=FALSE) {
-    stop(crayon::red(crayon::bold("UME model cannot be used for prediction")))
+  # Check whether UME has been used and stop if criteria for it are not met
+  if (!(FALSE %in% object[["model.arg"]][["UME"]])) {
+
+    if (length(treats)!=2) {
+      stop(crayon::red(crayon::bold("UME model can only be used for prediction of direct estimates for a single treatment comparison: length(treats) must equal 2")))
+    }
+
+    object <- prepare.ume.predict(mbnma=object, treats=treats, level=level)
   }
 
   # Check ref.resp has been specified correctly
@@ -431,12 +446,19 @@ predict.mbnma <- function(object, times=seq(0, max(object$model.arg$jagsdata$tim
       if (any(treats > NT | any(treats<1))) {
         stop(crayon::red(crayon::bold("If given as numeric treatment/class codes, `treats` must be numbered similarly to treatment/class codes in `object`")))
       }
+
+      # Sort treats into order in object$network
+      treats <- sort(treats)
+
       treats <- object$network[[level]][treats]
     }
     if (is.character(treats)) {
       if (!all(treats %in% object$network[[level]])) {
         stop(crayon::red(crayon::bold("`treats` includes treatments/classes not included in `object`")))
       }
+
+      # Sort treats into order in object$network
+      treats <- object$network[[level]][sort(match(treats, object$network[[level]]))]
     }
   }
 
@@ -446,7 +468,7 @@ predict.mbnma <- function(object, times=seq(0, max(object$model.arg$jagsdata$tim
   }
 
   # Check that distribution for E0 is of the correct format
-  if (class(E0)=="formula") {
+  if (inherits(E0,"formula")) {
     E0 <- as.character(E0)[2]
     if (grepl("r[A-z]+\\(n,.+\\)", E0)==FALSE) {
       stop(crayon::red(crayon::bold("Stochastic distribution for E0 must be expressed as a string in the form of a supported R distribution (e.g. '~rnorm(n, 5,2)')")))
@@ -459,6 +481,19 @@ predict.mbnma <- function(object, times=seq(0, max(object$model.arg$jagsdata$tim
     stop(crayon::red(crayon::bold("'E0' has been incorrectly specified")))
   }
 
+  # Ensure prediction intervals are used where appropriate
+  if (lim=="pred" & "random" %in% object$model.arg$a.method) {
+
+    addsd <- TRUE
+
+    if (!any(grepl("sd\\.", object$parameters.to.save))) {
+      stop(crayon::red("'sd' for time-course parameters not included in parameters.to.save...\n...cannot calculate prediction intervals"))
+    }
+
+    message("Bayesian prediction intervals to be calculated")
+  } else {
+    addsd <- FALSE
+  }
 
   ###### Extract info from mbnma #######
 
@@ -471,7 +506,7 @@ predict.mbnma <- function(object, times=seq(0, max(object$model.arg$jagsdata$tim
   # timecourse <- paste0("alpha + ", object$model.arg$fun$jags)
 
   # Extract parameter values from MBNMA result
-  model.vals <- get.model.vals(mbnma=object, E0=E0, level=level)
+  model.vals <- get.model.vals(mbnma=object, E0=E0, level=level, lim=lim)
   timecourse <- model.vals[["timecourse"]]
   time.params <- model.vals[["time.params"]]
 
@@ -546,9 +581,15 @@ predict.mbnma <- function(object, times=seq(0, max(object$model.arg$jagsdata$tim
   }
 
   # Convert predicted times to splines
-  if (any(c("rcs", "bs", "ns", "ls") %in% object$model.arg$fun)) {
+  if (any(c("bs", "ns", "ls") %in% object$model.arg$fun)) {
+
     timecourse <- gsub("\\[i\\,", "[", timecourse)
-    spline <- genspline(times, spline=object$model.arg$fun$name, knots=object$model.arg$fun$knots, degree=object$model.arg$fun$degree)
+    spline <- genspline(times, spline=object$model.arg$fun$name, knots=object$model.arg$fun$knots, degree=object$model.arg$fun$degree,
+                        boundaries = c(0, max(object$model.arg$jagsdata$time, na.rm=TRUE)))
+  }
+
+  if ("itp" %in% object$model.arg$fun$name) {
+    maxtime <- max(c(max(object$model.arg$jagsdata$time, na.rm=TRUE), times))
   }
 
   ########## Predict responses ###########
@@ -625,7 +666,7 @@ predict.mbnma <- function(object, times=seq(0, max(object$model.arg$jagsdata$tim
 
   #predict.result <- list("summary"=sumpred, "pred.mat"=predicts, "treatments"=object$network$treatments, "mbnma"=object)
   predict.result <- list("summary"=sumpred, "pred.mat"=predicts, "network"=object$network,
-                         "times"=times, "link"=object$model.arg$link)
+                         "times"=times, "link"=object$model.arg$link, "lim"=lim)
   class(predict.result) <- "mb.predict"
 
   return(predict.result)
@@ -837,13 +878,17 @@ plot.mbnma <- function(x, params=NULL, treat.labs=NULL, class.labs=NULL, ...) {
 #'
 #' Uses mbnma time-course parameter estimates to calculate treatment
 #' differences between treatments or classes at a particular time-point.
-#' Can be used to compare treatments evaluated in studies at different follow-up times.
+#' Can be used to compare treatments evaluated in studies at different follow-up times, or even
+#' to compare treatments in different MBNMA models via a common comparator.
 #'
 #' @param time A numeric value for the time at which to estimate relative effects/mean differences.
-#' @param treats A character vector of treatment names for which to calculate relative effects/mean differences.
-#' Must be a subset of `mbnma$network$treatments`
-#' @param classes A character vector of class names for which to calculate relative effects/mean differences from.
-#' Must be a subset of `mbnma$network$classes`. Only works for class effect models.
+#' @param treats A character vector of treatment names for which to calculate relative effects/mean
+#'   differences. Must be a subset of `mbnma$network$treatments`.
+#' @param classes A character vector of class names for which to calculate relative effects/mean
+#'   differences. Must be a subset of `mbnma$network$classes`. Only works for class effect models.
+#' @param mbnma.add An S3 object of `class("mbnma")` generated by running
+#'   a time-course MBNMA model. This should only be specified if results from two different MBNMA models
+#'   are to be combined to perform a 2-stage MBNMA (see Details).
 #' @inheritParams predict.mbnma
 #' @inheritParams fitplot
 #'
@@ -856,6 +901,19 @@ plot.mbnma <- function(x, params=NULL, treat.labs=NULL, class.labs=NULL, ...) {
 #'
 #' Results are reported in tables as the row-defined treatment minus the column-defined treatment.
 #'
+#' @details
+#' `get.relative()` can also be used to perform a 2-stage MBNMA that allows synthesis of results
+#' from two different MBNMA models via a single common comparator.
+#' In an MBNMA model, all treatments must share the same time-course function. However, a 2-stage
+#' approach can enable fitting of different time-course functions to different sets ("subnetworks") of
+#' treatments. For example, some treatments may have rich time-course information,
+#' allowing for a more complex time-course function to be used, whereas others may be sparse,
+#' requiring a simpler time-course function.
+#'
+#' Relative comparisons between treatments in the two datasets at specific follow-up times
+#' can then be estimated from MBNMA predicted effects versus a common comparator
+#' using the Bucher method and assuming consistency. See the MBNMAtime vignette for further details.
+#'
 #' @examples
 #' \donttest{
 #' # Create an mb.network object from a dataset
@@ -875,43 +933,175 @@ plot.mbnma <- function(x, params=NULL, treat.labs=NULL, class.labs=NULL, ...) {
 #' # Calculate difference between a subset of treatments at 10 weeks follow-up
 #' subres <- get.relative(mbnma, time=10,
 #'   treats=c("alog_50", "alog_25", "placebo"))
+#'
+#'
+#'
+#' ###########################
+#' ##### 2-stage MBNMA #####
+#' ###########################
+#'
+#' # Using the osteoarthritis dataset
+#' # With placebo (Pl_0) as common comparator between subnetworks
+#'
+#' #### Sparse model ####
+#'
+#' # Treatments on which time-course data is limited
+#' sparse.trt <- c("Ce_100", "Ce_400", "Du_90", "Lu_200", "Lu_400",
+#'   "Lu_NA", "Et_5", "Ox_44")
+#'
+#' # Create a subnetwork of studies comparing these treatments
+#' sparse.df <- osteopain %>% dplyr::group_by(studyID) %>%
+#'   dplyr::filter(any(treatment %in% sparse.trt)) %>%
+#'   dplyr::ungroup() %>%
+#'   subset(treatment %in% c("Pl_0", sparse.trt))
+#'
+#' sparse.net <- mb.network(sparse.df)
+#'
+#' # Run a ITP MBNMA with a known rate
+#' sparse.mbnma <- mb.run(sparse.net, fun=titp(method.rate=0.8, pool.rate="abs"))
+#'
+#'
+#' #### Complex model ####
+#'
+#' # Treatments on which time-course data is rich
+#' rich.trt <- levels(osteopain$treatment)[!levels(osteopain$treatment) %in%
+#'   c("Pl_0", "Ce_100", "Ce_400", "Du_90", "Lu_200",
+#'     "Lu_400", "Lu_NA", "Et_5", "Ox_44")]
+#'
+#' # Create a subnetwork of studies comparing these treatments
+#' rich.df <- osteopain %>% dplyr::group_by(studyID) %>%
+#'   dplyr::filter(any(treatment %in% rich.trt)) %>%
+#'   dplyr::ungroup() %>%
+#'   subset(treatment %in% c("Pl_0", rich.trt))
+#'
+#' rich.net <- mb.network(rich.df)
+#'
+#' # Run a Emax MBNMA
+#' rich.mbnma <- mb.run(rich.net, temax(p.expon = FALSE))
+#'
+#'
+#' #### Calculate relative effects between models ####
+#'
+#' # At 10 weeks follow-up
+#' rels.sparse <- get.relative(sparse.mbnma, time=10)
+#' rels.rich <- get.relative(rich.mbnma, time=10)
+#'
+#' rels.all <- get.relative(mbnma=rich.mbnma,
+#'   mbnma.add=sparse.mbnma, time=10)
+#'
+#' print(rels.all$median)
+#'
+#'
 #' }
 #' @export
-get.relative <- function(mbnma, time=max(mbnma$model.arg$jagsdata$time, na.rm=TRUE),
-                         treats=mbnma$network$treatments, classes=NULL) {
+get.relative <- function(mbnma, mbnma.add=NULL, time=max(mbnma$model.arg$jagsdata$time, na.rm=TRUE),
+                         treats=unique(c(mbnma$network$treatments, mbnma.add$network$treatments)),
+                         classes=NULL, lim="cred") {
 
   # Run checks
   argcheck <- checkmate::makeAssertCollection()
   checkmate::assertNumeric(time, lower=0, len=1, null.ok=FALSE, add=argcheck)
-  checkmate::assertSubset(treats, choices=mbnma$network$treatments, add=argcheck)
-  checkmate::assertSubset(classes, choices=mbnma$network$classes, add=argcheck)
+  checkmate::assertSubset(treats, choices=c(mbnma$network$treatments,
+                                            mbnma.add$network$treatments),
+                          add=argcheck)
+  checkmate::assertSubset(classes, choices=c(mbnma$network$classes,
+                                             mbnma.add$network$classes),
+                          add=argcheck)
+  checkmate::assertClass(mbnma, classes="mbnma", add=argcheck)
+  checkmate::assertClass(mbnma.add, classes="mbnma", null.ok = TRUE, add=argcheck)
   checkmate::reportAssertions(argcheck)
 
   if (!is.null(classes)) {
     treats <- classes
     level <- "class"
+    levels <- "classes"
   } else {
     level <- "treatment"
+    levels <- "treatments"
   }
 
-  pred <- suppressMessages(predict.mbnma(mbnma, times=time,
-                        treats = treats, level = level))
+  if (is.null(mbnma.add)) {
+    pred <- suppressMessages(predict.mbnma(mbnma, times=time,
+                                           treats = treats, level = level, lim=lim))
 
+    # Matrix of results
+    mat <- do.call(cbind, pred$pred.mat)
+
+  } else if (!is.null(mbnma.add)) {
+
+    # Identify common treatment to be the reference
+    match1 <- which(treats %in% mbnma$network[[levels]])
+    match2 <- which(treats %in% mbnma.add$network[[levels]])
+
+    ind1 <- which(match1 %in% match2)
+    ind2 <- which(match2 %in% match1)
+
+    ref <- treats[match1[ind1]]
+
+    if (length(ind1)!=1 | length(ind2)!=1) {
+      stop("mbnma and mbnma.add must have a single treatment/class in common in order to perform 2-stage MBNMA")
+    }
+
+    # Split treatments/classes into subnetworks
+    treats2 <- c(ref, treats[!treats %in% mbnma$network[[levels]]])
+    treats1 <- c(ref, treats[!treats %in% mbnma.add$network[[levels]]])
+
+    # Predict relative effects at specific time
+    pred <- suppressMessages(predict.mbnma(mbnma, times=time,
+                                           treats = treats1, level = level, lim=lim))
+
+    pred.add <- suppressMessages(predict.mbnma(mbnma.add, times=time,
+                                               treats = treats2, level = level, lim=lim))
+
+    # Matrix of results
+    mat1 <- do.call(cbind, pred$pred.mat)
+    mat2 <- do.call(cbind, pred.add$pred.mat)
+
+    # Ensure matrices are same size
+    niter <- min(c(nrow(mat1), c(nrow(mat2))))
+
+    if (nrow(mat1)>nrow(mat2)) {
+      mat1 <- mat1[sample(1:nrow(mat1), nrow(mat2), replace = FALSE),]
+    } else if (nrow(mat1)<nrow(mat2)) {
+      mat2 <- mat2[sample(1:nrow(mat2), nrow(mat1), replace = FALSE),]
+    }
+
+    # Rearrange matrix to common reference treatment
+    if (which(names(pred$summary) %in% ref)!=1) {
+      mat1 <- mat1 - mat1[,ind1]
+
+      # Rearrange columns
+      refcol <- which(names(pred$summary) %in% ref)
+      mat1 <- cbind(mat1[,refcol], mat1[,-refcol])
+    }
+    if (which(names(pred.add$summary) %in% ref)!=1) {
+      mat2 <- mat2 - mat2[,ind2]
+
+      # Rearrange columns
+      refcol <- which(names(pred.add$summary) %in% ref)
+      mat2 <- cbind(mat2[,refcol], mat2[,-refcol])
+    }
+
+    # Drop shared reference from mat2
+    mat2 <- mat2[,-which(treats2 %in% ref)]
+
+    # Combine results
+    mat <<- cbind(mat1, mat2)
+    treats <- c(treats1, treats2[-which(treats2 %in% ref)])
+  }
 
-  # Matrix of results
-  mat <- do.call(cbind, pred$pred.mat)
 
   # For lower triangle
   outmat <- array(dim=c(length(treats), length(treats), nrow(mat)))
   for (i in 1:(ncol(mat)-1)) {
-    temp <- mat[,-i] - mat[,i]
+    temp <- mat[,-i, drop=FALSE] - mat[,i]
     #temp <- apply(temp, MARGIN=2, FUN=function(x) {neatCrI(quantile(x, probs=c(0.025, 0.5, 0.975)), digits = 2)})
     outmat[(1+i):dim(outmat)[1],i,] <- t(temp[,i:ncol(temp)])
   }
 
   # For upper triangle
   for (i in 1:(ncol(mat)-1)) {
-    temp <- mat[,i] - mat[,-i]
+    temp <- mat[,i] - mat[,-i, drop=FALSE]
     outmat[i,(1+i):dim(outmat)[2],] <- t(temp[,i:ncol(temp)])
   }
 
@@ -929,6 +1119,7 @@ get.relative <- function(mbnma, time=max(mbnma$model.arg$jagsdata$time, na.rm=TR
   medmat <- meanmat
   l95mat <- medmat
   u95mat <- medmat
+  sum.df <- data.frame(comparison=NA, mean=NA, sd=NA, l95=NA, med=NA, u95=NA)
 
   for (i in 1:nrow(xmat)) {
     for (k in 1:ncol(xmat)) {
@@ -938,19 +1129,32 @@ get.relative <- function(mbnma, time=max(mbnma$model.arg$jagsdata$time, na.rm=TR
         medmat[i,k] <- stats::median(xmat[i,k,])
         l95mat[i,k] <- stats::quantile(xmat[i,k,], probs = 0.025)
         u95mat[i,k] <- stats::quantile(xmat[i,k,], probs = 0.975)
+
+        if (k>i) {
+          sum.df <- dplyr::add_row(sum.df,
+                                   comparison = paste0(treats[i], " vs ", treats[k]),
+                                   mean=meanmat[i,k],
+                                   sd=semat[i,k],
+                                   l95=l95mat[i,k],
+                                   med=medmat[i,k],
+                                   u95=u95mat[i,k]
+          )
+        }
       }
     }
   }
+  sum.df <- sum.df[-1,]
+  names(sum.df) <- c("comparison", "mean", "sd", "2.5%", "50%", "97.5%")
 
-  sumlist <- list("mean"=meanmat, "se"=semat, "median"=medmat, "lower95"=l95mat, "upper95"=u95mat)
+  sumlist <- list("mean"=meanmat, "sd"=semat, "median"=medmat, "lower95"=l95mat, "upper95"=u95mat)
 
   for (i in seq_along(sumlist)) {
     dimnames(sumlist[[i]])[[1]] <- dimnames(xmat)[[1]]
     dimnames(sumlist[[i]])[[2]] <- dimnames(xmat)[[2]]
   }
 
-  out <- list("time"=time, "relarray"=outmat)
-  out <- c(out, sumlist)
+  out <- list("time"=time, "data.frame"=sum.df, "relarray"=outmat)
+  out <- c(out, sumlist, lim=lim)
 
   class(out) <- "relative.array"
 
